@@ -30,7 +30,14 @@ from ._admin import (
     _Cluster,
     _ClusterUser,
     _ClusterUserRoleType,
+    _ClusterUserWithInfo,
+    _ConfigCluster,
     _NodePool,
+    _Org,
+    _OrgCluster,
+    _OrgUser,
+    _OrgUserRoleType,
+    _OrgUserWithInfo,
     _Quota,
     _Storage,
     _UserInfo,
@@ -102,7 +109,7 @@ from ._url_utils import CLUSTER_SCHEMES as SCHEMES
 from ._users import Action, Permission, Quota, Share, Users
 from ._utils import _ContextManager, find_project_root
 
-__version__ = "21.11.4"
+__version__ = "21.12.1"
 
 
 __all__ = (
@@ -201,7 +208,14 @@ __all__ = (
     "_Cluster",
     "_ClusterUser",
     "_ClusterUserRoleType",
+    "_ClusterUserWithInfo",
+    "_ConfigCluster",
     "_NodePool",
+    "_Org",
+    "_OrgCluster",
+    "_OrgUser",
+    "_OrgUserRoleType",
+    "_OrgUserWithInfo",
     "_Quota",
     "_Storage",
     "_UserInfo",
