@@ -45,7 +45,7 @@ from ._images import (
     _try_parse_image_progress_step,
 )
 from ._parser import DiskVolume, Parser, SecretFile, Volume
-from ._parsing_utils import LocalImage, RemoteImage, _as_repo_str, _is_in_neuro_registry
+from ._parsing_utils import LocalImage, RemoteImage
 from ._rewrite import rewrite_module
 from ._url_utils import (
     normalize_disk_uri,
@@ -576,12 +576,12 @@ class Jobs(metaclass=NoPublicConstructor):
         progress: Optional[AbstractDockerImageProgress] = None,
         cluster_name: Optional[str] = None,
     ) -> None:
-        if not _is_in_neuro_registry(image):
+        if not image._is_in_neuro_registry:
             raise ValueError(f"Image `{image}` must be in the neuro registry")
         if progress is None:
             progress = _DummyProgress()
 
-        payload = {"container": {"image": _as_repo_str(image)}}
+        payload = {"container": {"image": image.as_docker_url()}}
         url = self._get_monitoring_url(cluster_name) / id / "save"
 
         auth = await self._config._api_auth()
@@ -918,7 +918,7 @@ def _container_to_api(
     tty: bool = False,
     shm: bool = False,
 ) -> Dict[str, Any]:
-    primitive: Dict[str, Any] = {"image": _as_repo_str(image)}
+    primitive: Dict[str, Any] = {"image": image.as_docker_url()}
     if shm:
         primitive["resources"] = {"shm": shm}
     if entrypoint:
@@ -935,7 +935,11 @@ def _container_to_api(
         primitive["volumes"] = [_volume_to_api(v, config) for v in volumes]
     if secret_env:
         primitive["secret_env"] = {
-            k: str(normalize_secret_uri(v, config.username, config.cluster_name))
+            k: str(
+                normalize_secret_uri(
+                    v, config.username, config.cluster_name, config.org_name
+                )
+            )
             for k, v in secret_env.items()
         }
     if secret_files:
@@ -1072,10 +1076,6 @@ def _job_to_api(
     if privileged:
         primitive["privileged"] = privileged
     if org_name:
-        log.warning(
-            "'org_name' parameter is not fully supported yet, "
-            "and will be ignored by the system!"
-        )
         primitive["org_name"] = org_name
     primitive["cluster_name"] = cluster_name
     return primitive
@@ -1093,7 +1093,7 @@ def _job_telemetry_from_api(value: Dict[str, Any]) -> JobTelemetry:
 
 def _volume_to_api(volume: Volume, config: Config) -> Dict[str, Any]:
     uri = normalize_storage_path_uri(
-        volume.storage_uri, config.username, config.cluster_name
+        volume.storage_uri, config.username, config.cluster_name, config.org_name
     )
     resp: Dict[str, Any] = {
         "src_storage_uri": str(uri),
@@ -1105,7 +1105,7 @@ def _volume_to_api(volume: Volume, config: Config) -> Dict[str, Any]:
 
 def _secret_file_to_api(secret_file: SecretFile, config: Config) -> Dict[str, Any]:
     uri = normalize_secret_uri(
-        secret_file.secret_uri, config.username, config.cluster_name
+        secret_file.secret_uri, config.username, config.cluster_name, config.org_name
     )
     return {
         "src_secret_uri": str(uri),
@@ -1114,7 +1114,9 @@ def _secret_file_to_api(secret_file: SecretFile, config: Config) -> Dict[str, An
 
 
 def _disk_volume_to_api(volume: DiskVolume, config: Config) -> Dict[str, Any]:
-    uri = normalize_disk_uri(volume.disk_uri, config.username, config.cluster_name)
+    uri = normalize_disk_uri(
+        volume.disk_uri, config.username, config.cluster_name, config.org_name
+    )
     resp: Dict[str, Any] = {
         "src_disk_uri": str(uri),
         "dst_path": volume.container_path,
