@@ -12,19 +12,19 @@ import pytest
 from aiohttp import web
 from yarl import URL
 
-import neuro_sdk.storage
 from neuro_sdk import (
     Action,
     Client,
+    DiskUsageInfo,
     FileStatus,
     FileStatusType,
     IllegalArgumentError,
     StorageProgressComplete,
+    StorageProgressDelete,
     StorageProgressStart,
     StorageProgressStep,
 )
-from neuro_sdk.abc import StorageProgressDelete
-from neuro_sdk.storage import DiskUsageInfo, _parse_content_range
+from neuro_sdk._storage import _parse_content_range
 
 from tests import _RawTestServerFactory, _TestServerFactory
 
@@ -50,7 +50,9 @@ def calc_diff(dcmp: "dircmp[str]", *, pre: str = "") -> List[Tuple[str, str]]:
 
 @pytest.fixture
 def small_block_size(monkeypatch: Any) -> None:
-    monkeypatch.setattr(neuro_sdk.storage, "READ_SIZE", 300)
+    import neuro_sdk._storage
+
+    monkeypatch.setattr(neuro_sdk._storage, "READ_SIZE", 300)
 
 
 @pytest.fixture
@@ -116,18 +118,23 @@ async def storage_server(
         elif op == "GETFILESTATUS":
             if not local_path.exists():
                 raise web.HTTPNotFound()
-            stat = local_path.stat()
-            return web.json_response(
-                {
-                    "FileStatus": {
-                        "path": local_path.name,
-                        "type": "FILE" if local_path.is_file() else "DIRECTORY",
-                        "length": stat.st_size,
-                        "modificationTime": stat.st_mtime,
-                        "permission": "write",
-                    }
-                }
-            )
+            stat = local_path.lstat()
+            status = {
+                "path": local_path.name,
+                "length": stat.st_size,
+                "modificationTime": stat.st_mtime,
+                "permission": "write",
+            }
+            if local_path.is_symlink():
+                status["type"] = "SYMLINK"
+                status["target"] = os.readlink(local_path)
+            elif local_path.is_file():
+                status["type"] = "FILE"
+            elif local_path.is_dir():
+                status["type"] = "DIRECTORY"
+            else:
+                status["type"] = "UNKNOWN"
+            return web.json_response({"FileStatus": status})
 
         elif op == "MKDIRS":
             try:
@@ -144,16 +151,23 @@ async def storage_server(
                 raise web.HTTPNotFound()
             ret = []
             for child in local_path.iterdir():
-                stat = child.stat()
-                ret.append(
-                    {
-                        "path": child.name,
-                        "type": "FILE" if child.is_file() else "DIRECTORY",
-                        "length": stat.st_size,
-                        "modificationTime": stat.st_mtime,
-                        "permission": "write",
-                    }
-                )
+                stat = child.lstat()
+                status = {
+                    "path": child.name,
+                    "length": stat.st_size,
+                    "modificationTime": stat.st_mtime,
+                    "permission": "write",
+                }
+                if child.is_symlink():
+                    status["type"] = "SYMLINK"
+                    status["target"] = os.readlink(local_path)
+                elif child.is_file():
+                    status["type"] = "FILE"
+                elif child.is_dir():
+                    status["type"] = "DIRECTORY"
+                else:
+                    status["type"] = "UNKNOWN"
+                ret.append(status)
             return await make_listiter_response(request, ret)
 
         else:
@@ -182,6 +196,21 @@ async def test_storage_ls_legacy(
                     "modificationTime": 0,
                     "permission": "read",
                 },
+                {
+                    "path": "baz",
+                    "length": 1,
+                    "type": "SYMLINK",
+                    "modificationTime": 0,
+                    "permission": "read",
+                    "target": "foo",
+                },
+                {
+                    "path": "spam",
+                    "length": 1,
+                    "type": "SPAM",
+                    "modificationTime": 0,
+                    "permission": "read",
+                },
             ]
         }
     }
@@ -214,6 +243,23 @@ async def test_storage_ls_legacy(
             permission=Action.READ,
             uri=URL("storage://default/user/folder/bar"),
         ),
+        FileStatus(
+            path="baz",
+            size=1,
+            type=FileStatusType.SYMLINK,
+            modification_time=0,
+            permission=Action.READ,
+            target="foo",
+            uri=URL("storage://default/user/folder/baz"),
+        ),
+        FileStatus(
+            path="spam",
+            size=1,
+            type=FileStatusType.UNKNOWN,
+            modification_time=0,
+            permission=Action.READ,
+            uri=URL("storage://default/user/folder/spam"),
+        ),
     ]
 
     async with make_client(srv.make_url("/")) as client:
@@ -258,6 +304,21 @@ async def test_storage_ls(
             "modificationTime": 0,
             "permission": "read",
         },
+        {
+            "path": "baz",
+            "length": 1,
+            "type": "SYMLINK",
+            "modificationTime": 0,
+            "permission": "read",
+            "target": "foo",
+        },
+        {
+            "path": "spam",
+            "length": 1,
+            "type": "SPAM",
+            "modificationTime": 0,
+            "permission": "read",
+        },
     ]
 
     async def handler(request: web.Request) -> web.StreamResponse:
@@ -292,6 +353,23 @@ async def test_storage_ls(
             permission=Action.READ,
             uri=URL("storage://default/user/folder/bar"),
         ),
+        FileStatus(
+            path="baz",
+            size=1,
+            type=FileStatusType.SYMLINK,
+            modification_time=0,
+            permission=Action.READ,
+            target="foo",
+            uri=URL("storage://default/user/folder/baz"),
+        ),
+        FileStatus(
+            path="spam",
+            size=1,
+            type=FileStatusType.UNKNOWN,
+            modification_time=0,
+            permission=Action.READ,
+            uri=URL("storage://default/user/folder/spam"),
+        ),
     ]
 
 
@@ -353,6 +431,21 @@ async def test_storage_ls_another_cluster(
             "modificationTime": 0,
             "permission": "read",
         },
+        {
+            "path": "baz",
+            "length": 1,
+            "type": "SYMLINK",
+            "modificationTime": 0,
+            "permission": "read",
+            "target": "foo",
+        },
+        {
+            "path": "spam",
+            "length": 1,
+            "type": "SPAM",
+            "modificationTime": 0,
+            "permission": "read",
+        },
     ]
 
     async def handler(request: web.Request) -> web.StreamResponse:
@@ -387,6 +480,23 @@ async def test_storage_ls_another_cluster(
             permission=Action.READ,
             uri=URL("storage://another/user/folder/bar"),
         ),
+        FileStatus(
+            path="baz",
+            size=1,
+            type=FileStatusType.SYMLINK,
+            modification_time=0,
+            permission=Action.READ,
+            target="foo",
+            uri=URL("storage://another/user/folder/baz"),
+        ),
+        FileStatus(
+            path="spam",
+            size=1,
+            type=FileStatusType.UNKNOWN,
+            modification_time=0,
+            permission=Action.READ,
+            uri=URL("storage://another/user/folder/spam"),
+        ),
     ]
 
 
@@ -1139,6 +1249,43 @@ async def test_storage_stats_another_cluster(
         )
 
 
+async def test_storage_stats_symlink(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.Response:
+        assert request.path == "/storage/user/link"
+        assert request.query == {"op": "GETFILESTATUS"}
+        return web.json_response(
+            {
+                "FileStatus": {
+                    "path": "/user/link",
+                    "type": "SYMLINK",
+                    "length": 1234,
+                    "modificationTime": 3456,
+                    "permission": "read",
+                    "target": "folder/subfolder/file",
+                }
+            }
+        )
+
+    app = web.Application()
+    app.router.add_get("/storage/user/link", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        stats = await client.storage.stat(URL("storage:link"))
+        assert stats == FileStatus(
+            path="/user/link",
+            type=FileStatusType.SYMLINK,
+            size=1234,
+            modification_time=3456,
+            permission=Action.READ,
+            target="folder/subfolder/file",
+            uri=URL("storage://default/user/link"),
+        )
+
+
 async def test_storage_open(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -1692,7 +1839,9 @@ async def test_storage_download_dir_slash_ending(
 
 @pytest.fixture
 def zero_time_threshold(monkeypatch: Any) -> None:
-    monkeypatch.setattr(neuro_sdk.storage, "TIME_THRESHOLD", 0.0)
+    import neuro_sdk._storage
+
+    monkeypatch.setattr(neuro_sdk._storage, "TIME_THRESHOLD", 0.0)
 
 
 async def test_storage_upload_file_update(
